/* Copyright 2008-2017 Research Foundation State University of New York   */

/* This file is part of QUB Online.                                       */

/* QUB Online is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License as published by   */
/* the Free Software Foundation, either version 3 of the License, or      */
/* (at your option) any later version.                                    */

/* QUB Online is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License,     */
/* named LICENSE.txt, in the QUB Online program directory.  If not, see  */
/* <http://www.gnu.org/licenses/>.                                        */

#ifndef QUB_EVENTMAKER_H
#define QUB_EVENTMAKER_H

#include <vector>
using std::vector;
#include <map>
using std::map;

#include "qubfast.h"
#include "qub_mean.h"
#include "QUB_Heap.h"

// find nearest amp in list of known amps and return whether it's within delta
int FindKnownAmp(vector<double>& known, double amp, double delta, double& ampOut);

class EventMaker_Cls
{
public:
  double amp, std;
  int count;

  EventMaker_Cls()
  {}
  EventMaker_Cls(double _amp, double _std, int _count)
    : amp(_amp), std(_std), count(_count)
  {}
  void set(double _amp, double _std, int _count)
  {
    amp = _amp;
    std = _std;
    count = _count;
  }
};

class EventMaker
{
public:
	vector<int> clss;
	vector<int> counts;
	map<double, int> clsOfAmp;
	vector<double> ampOfCls;
	int at;

	EventMaker()
		: at(0)
	{}

	// stage one: add all your events
	void add(double amp, int count);
	// stage two: consolidate
	void consolidate(double delta, vector<double>& known, int& StopFlag);
	// stage three: pull out finished events
	int nextEvent(int& cls, int& count, int maxCount); // returns count
	int findOrAddCls(double amp);
};

class QUB_EventMaker_ByGauss {
private:
  std::vector<IncrementalRSS> dist;
  std::map<double, int> byMean;
  std::vector<int> mean_order;
  std::vector<int> rewrite;
  std::vector<int> clss;
  std::vector<int> counts;
  int at;

  bool exists(int c) { return (rewrite[c] == c); }
  int get_mean_index(int c);
  void combine(int c, int d);
  bool combine_if_within_std(int c, int d, double delta);
  bool combine_neighbor(int c, double delta);
  int indices_by_size_dec(vector<int>& indices);
  void renumber(int *cc, int n);
  void renumber_classes();
  int nextEvent(int& cls, int& count, int maxCount); // returns count
  
public:
 QUB_EventMaker_ByGauss()
   : at(0)
    {}
  
  int add_event(double mean, double rss, int n);
  void done_add();
  void combine_all(double delta);
  int get_next_dwells(int offset_in_data, int sample_count, int *firsts, int *lasts, int *classes);
  int class_count();
  void get_dist(double *mean, double *std);
};
 
int delete_short_events(int ndwt, int *firsts, int *lasts, int *classes, int min_nsamp);
int delete_unused_classes(int ndwt, int *classes, int nc, float *mean, float *std);

#endif
